#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Provide helper parameter values."""

import configparser

"""Provide helper parameter values."""
# Read parameters from configuration file.
config = configparser.ConfigParser(allow_no_value=True)
config.read('parameters.ini')

# Execution parameters.
DRY_RUN = config.getboolean('execution', 'DRY_RUN')
THREAD_NUM = config.getint('execution', 'THREAD_NUM')

# AWS entrypoint parameters.
auth_method = config.get('entrypoint', 'auth_method')
profile_name = config.get('entrypoint', 'profile_name')
access_key_id = config.get('entrypoint', 'access_key_id')
secret_access_key = config.get('entrypoint', 'secret_access_key')
sso_url = config.get('entrypoint', 'sso_url')
sso_role_name = config.get('entrypoint', 'sso_role_name')
sso_account_id = config.get('entrypoint', 'sso_account_id')
accounts_list_src = config.get('entrypoint', 'accounts_list_src')

# AWS environment parameters.
region = config.get('environment', 'region')
assumed_role_name = config.get('environment', 'assumed_role_name')
external_id = config.get('environment', 'external_id')
sso_instance_arn = config.get('environment', 'sso_instance_arn')
identity_store_id = config.get('environment', 'identity_store_id')

# AWS accounts parameters.
accounts = config.get('accounts', 'accounts')
ou = config.get('accounts', 'ou')
statuses = config.get('accounts', 'statuses')

# actions_get script parameters.
actions_get_output_file = config.get('actionsget', 'actions_get_output_file')
actions_get_service = config.get('actionsget', 'actions_get_service')
actions_get_filter = config.get('actionsget', 'actions_get_filter')

# sgs_get script parameters.
sgs_get_output_file = config.get('sgsget', 'sgs_get_output_file')

# sso_get script parameters.
sso_get_output_file = config.get('ssoget', 'sso_get_output_file')

# sso_json script parameters.
sso_json_input_file = config.get('ssojson', 'sso_json_input_file')
sso_json_output_file = config.get('ssojson', 'sso_json_output_file')

# sso_mappings script parameters.
sso_mappings_map_file = config.get('ssomappings', 'sso_mappings_map_file')
sso_mappings_input_file = config.get('ssomappings', 'sso_mappings_input_file')
sso_mappings_output_file = config.get('ssomappings', 'sso_mappings_output_file')
sso_mappings_old_user_heading = config.get('ssomappings', 'sso_mappings_old_user_heading')
sso_mappings_new_user_heading = config.get('ssomappings', 'sso_mappings_new_user_heading')
sso_mappings_old_group_pattern = config.get('ssomappings', 'sso_mappings_old_group_pattern')
sso_mappings_new_group_pattern = config.get('ssomappings', 'sso_mappings_new_group_pattern')

# sso_assign script parameters.
sso_assign_input_file = config.get('ssoassign', 'sso_assign_input_file')

# sso_update script parameters.
sso_update_user_map_file = config.get('ssoupdate', 'sso_update_user_map_file')
sso_update_group_map_file = config.get('ssoupdate', 'sso_update_group_map_file')
sso_update_old_user_heading = config.get('ssoupdate', 'sso_update_old_user_heading')
sso_update_new_user_heading = config.get('ssoupdate', 'sso_update_new_user_heading')
sso_update_old_group_heading = config.get('ssoupdate', 'sso_update_old_group_heading')
sso_update_new_group_heading = config.get('ssoupdate', 'sso_update_new_group_heading')

# sso_memberships script parameters.
sso_memberships_input_file = config.get('ssomemberships', 'sso_memberships_input_file')
sso_memberships_output_file = config.get('ssomemberships', 'sso_memberships_output_file')

# s3_getbuckets script parameters.
s3_getbuckets_output_file = config.get('s3getbuckets', 's3_getbuckets_output_file')
s3_getbuckets_prefix = config.get('s3getbuckets', 's3_getbuckets_prefix')

# vpc_getvpcs script parameters.
vpc_getvpcs_output_file = config.get('vpcgetvpcs', 'vpc_getvpcs_output_file')



def get_permission_sets(account_id: str, items: dict) -> dict: ...
def read_users(data: list) -> list: ...
def read_groups(data: list) -> list: ...
def read_permission_sets(data: list) -> list: ...
def get_all_permission_set_arns(sso_admin, sso_instance_arn: str) -> list: ...
def get_user_id(user_name: str, items: dict) -> dict: ...
def get_group_id(group_name: str, items: dict) -> dict: ...
def get_permission_set_arn(permission_set_name: str, items: dict) -> dict: ...
def process_json(file_name: str, user_ids: list, group_ids: list, permission_set_arns: list) -> str: ...
def process_mappings(data: list, file_name: str, sso_mappings_old_user_heading: str, sso_mappings_new_user_heading: str, sso_mappings_old_group_pattern: str, sso_mappings_new_group_pattern: str) -> str: ...
def assign_permission_sets(account_id: str, items: dict) -> list: ...
def assign_permission_sets_dry_run(account_id: str, items: dict) -> list: ...
def update_user_name(user_dict: dict, items: dict) -> dict: ...
def update_group_name(group_dict: dict, items: dict) -> dict: ...
def get_group_memberships(group_name: str, items: dict) -> dict: ...
def main() -> None: ...



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""
List S3 buckets in AWS accounts.

The bucket name prefix and region can be provided to tind specific buckets.
"""

from . import helper_aws_s3
from . import helper_common
from . import helper_aws_entrypoint
from .helper_parameters import *  # NOQA
from multithreader import threads


def aws_s3_getbuckets():
    """Get S3 buckets."""
    # Enter AWS environment.
    aws = helper_aws_entrypoint.enter_aws()

    # Create items for multithreading.
    items = {
        'session': aws['session'],
        'assumed_role_name': assumed_role_name,
        'external_id': external_id,
        's3_getbuckets_prefix': s3_getbuckets_prefix,
        'region': region
    }

    # Execute task with multithreading.
    buckets = threads(
        helper_aws_s3.get_buckets,
        aws['account_ids'],
        items,
        thread_num=THREAD_NUM
    )

    # Write the list to JSON file.
    helper_common.write_json_obj(s3_getbuckets_output_file, buckets)


def main():
    """Execute main function."""
    aws_s3_getbuckets()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: latin-1 -*-

"""
Run AWS administrative scripts.

Prerequisite steps:
1. Copy the extras/parameters.ini file to your current working directory.
2. Update the file with the necessary values.

Example usage:
```python
from aws_administrator import aws_sso_get
aws_sso_get.aws_sso_get()
```
"""
import __init__



from .helper_parameters import *
from . import helper_aws_entrypoint as helper_aws_entrypoint, helper_aws_s3 as helper_aws_s3, helper_common as helper_common

def aws_s3_getbuckets() -> None: ...
def main() -> None: ...



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Provide common helper functions."""

import sys
import csv
import json
import datetime
import logging
from typing import Union, Dict, List


__application__ = "aws_administrator_common_helper"


def get_current_time() -> str:
    """Get current date and time in UTC."""
    return datetime.datetime.now(datetime.UTC).strftime('%Y-%m-%d_%H-%M-%S')


def exception_handler(type, value, tb) -> None:
    """Handle uncaught exceptions."""
    logger = logging.getLogger(f'{__name__}')
    logging.basicConfig(
        filename=f'/tmp/{__application__}.{get_current_time()}.log',
        level=logging.WARNING
    )
    log_level = 30
    # NOTSET (0), DEBUG (10), INFO (20), WARNING (30), ERROR (40), CRITICAL (50), NONE (100)
    logger.setLevel(log_level)

    error_message = (
        f'{type.__name__}: {tb.tb_frame.f_code.co_name} '
        f'at line {tb.tb_lineno} in {tb.tb_frame.f_code.co_filename}: '
        f'{str(value)}'
    )
    print(error_message)
    logger.error(f'{get_current_time()}:Exception: {str(error_message)}')


def read_csv(
    file_name: str
) -> list:
    """Read CSV file and convert to dictionaries."""
    with open(file_name, 'r') as f:
        reader = csv.DictReader(f)
        data = [row for row in reader]
    return data


def dicts_to_csv(
    data: list
) -> list:
    """Convert dicts data to CSV list."""
    csv_data = [list(data[0].keys())]
    for row in data:
        csv_data.append(list(row.values()))
    return csv_data


def export_csv(
    data: list,
    file: str
) -> None:
    """Export data to CSV file."""
    with open(file, 'w') as f:
        writer = csv.writer(f)
        writer.writerows(data)


def read_json(
    file_name: str
) -> dict:
    """Read JSON file."""
    with open(file_name, 'r') as f:
        data = json.load(f)
    return data


def write_json_obj(
    file_name: str,
    data: Union[Dict, List]
) -> None:
    """Write JSON to file."""
    with open(file_name, 'w') as f:
        json.dump(data, f, indent=4, default=str)


def write_json_str(
    file_name: str,
    data: str
) -> None:
    """Write JSON strings to file."""
    with open(file_name, 'w') as f:
        f.write(data)


def get_name_from_tags(
    tags: list
) -> str:
    """Get Name tag value from a list of tags."""
    name = ''
    for tag in tags:
        if tag['Key'] == 'Name':
            name = tag['Value']
            break
    return name


def main():
    """Execute main function."""
    pass


sys.excepthook = exception_handler

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Convert human-readable SSO data to AWS IDs."""

from . import helper_aws_sso
from . import helper_common
from . import helper_aws_entrypoint
from .helper_parameters import *  # NOQA
from multithreader import threads


def aws_sso_json():
    """Convert human-readable SSO data to AWS IDs."""
    # Enter AWS environment.
    aws = helper_aws_entrypoint.enter_aws()
    sso_admin = aws['session'].client(
        'sso-admin',
        region_name=region
    )
    identity_store = aws['session'].client(
        'identitystore',
        region_name=region
    )

    # List all Permission Sets in the organization.
    permission_sets = helper_aws_sso.get_all_permission_set_arns(
        sso_admin,
        sso_instance_arn
    )

    # Read JSON file.
    data = helper_common.read_json(sso_json_input_file)
    details = [item['details'] for item in data]

    # Read Permission Sets and principals from dictionary.
    permission_set_names = helper_aws_sso.read_permission_sets(details)
    users = helper_aws_sso.read_users(details)
    groups = helper_aws_sso.read_groups(details)

    # Execute tasks with multithreading.
    items = {
        'identity_store': identity_store,
        'identity_store_id': identity_store_id,
        'sso_admin': sso_admin,
        'sso_instance_arn': sso_instance_arn,
        'permission_sets': permission_sets
    }
    user_ids = threads(
        helper_aws_sso.get_user_id,
        users,
        items,
        thread_num=THREAD_NUM
    )
    group_ids = threads(
        helper_aws_sso.get_group_id,
        groups,
        items,
        thread_num=THREAD_NUM
    )
    permission_set_arns = threads(
        helper_aws_sso.get_permission_set_arn,
        permission_set_names,
        items,
        thread_num=THREAD_NUM
    )

    # Process JSON file.
    processed = helper_aws_sso.process_json(
        sso_json_input_file,
        user_ids,
        group_ids,
        permission_set_arns
    )

    # Write results to file.
    helper_common.write_json_str(sso_json_output_file, processed)


def main():
    """Execute main function."""
    aws_sso_json()


if __name__ == '__main__':
    main()



from .helper_parameters import *
from . import helper_aws_ec2 as helper_aws_ec2, helper_aws_entrypoint as helper_aws_entrypoint, helper_common as helper_common

def aws_vpc_getvpcs() -> None: ...
def main() -> None: ...



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Provide common AWS S3 multithreaded functions."""

from . import helper_aws_entrypoint


def get_buckets(
    account_id: str,
    items: dict
) -> dict:
    """
    List S3 buckets in an AWS account.

    The bucket name prefix and region can be provided to tind specific buckets.
    """
    try:
        # Get auth credentials for each account.
        client = helper_aws_entrypoint.account_client_auth(
            account_id,
            items,
            's3'
        )

        # Create the paginator and get the list of buckets.
        paginator = client.get_paginator('list_buckets')

        # Get data based on the prefix and region.
        if not items['s3_getbuckets_prefix'] and not items['region']:
            response_iterator = paginator.paginate()
        elif not items['s3_getbuckets_prefix']:
            response_iterator = paginator.paginate(
                BucketRegion=items['region']
            )
        elif not items['region']:
            response_iterator = paginator.paginate(
                Prefix=items['s3_getbuckets_prefix']
            )
        else:
            response_iterator = paginator.paginate(
                Prefix=items['s3_getbuckets_prefix'],
                BucketRegion=items['region']
            )

        # Read through pages of bucket details and get bucket names.
        buckets = []
        for page in response_iterator:
            for bucket in page['Buckets']:
                buckets.append(bucket['Name'])

    except Exception as e:
        buckets = [str(e)]

    return {
        'account_id': account_id,
        'buckets': buckets
    }


def main():
    """Execute main function."""
    pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Get AWS actions for services."""

from pprint import pprint as pp
from . import helper_aws_iam
from . import helper_common
from . import helper_aws_entrypoint
from .helper_parameters import *  # NOQA


def aws_actions_get():
    """Get AWS actions for services."""
    # Enter AWS environment.
    aws = helper_aws_entrypoint.enter_aws()

    # Get actions.
    actions = helper_aws_iam.get_actions(
        aws['session'],
        actions_get_service
    )

    # Write policies to JSON file.
    helper_common.write_json_obj(actions_get_output_file, actions)

    # Filter actions by keyword.
    if actions_get_filter:
        filtered_actions = helper_aws_iam.filter_actions_keyword(
            actions,
            actions_get_filter
        )
        pp(filtered_actions)


def main():
    """Execute main function."""
    aws_actions_get()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Provide common AWS IAM multithreaded functions."""


def get_actions(
    session,
    service: str
) -> list:
    """
    Get AWS actions by service name.

    Ref: https://stackoverflow.com/questions/51930111/boto3-get-available-actions-per-service

    This is not 100% comprehensive. For example, it doesn't include "iam:PassRole".
    """
    actions = []

    match service:

        # If service is not specified, get all services.
        case None:

            # For non-IAM services.
            for service in session.get_available_services():
                service_client = session.client(service)
                actions.append(
                    {
                        "service": service,
                        "actions": service_client.meta.service_model.operation_names
                    }
                )

            # For IAM.
            iam = session.client('iam')
            actions.append(
                {
                    "service": "iam",
                    "actions": iam.meta.service_model.operation_names
                }
            )

        # Get actions for iam only.
        case 'iam':
            iam = session.client('iam')
            actions.append(
                {
                    "service": "iam",
                    "actions": iam.meta.service_model.operation_names
                }
            )

        # Get actions for specified service.
        case _:
            service_client = session.client(service)
            actions.append(
                {
                    "service": service,
                    "actions": service_client.meta.service_model.operation_names
                }
            )

    return actions


def filter_actions_keyword(
    data: list,
    keyword: str
) -> list:
    """Filter AWS actions by keyword."""
    filtered_actions = []
    for service in data:
        for action in service['actions']:
            if keyword in action:
                filtered_actions.append(f'{service['service']}:{action}')
    return filtered_actions


def main():
    """Execute main function."""
    pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Update AWS SSO User and Group names from mappings files."""

from pprint import pprint as pp
from . import helper_aws_sso
from . import helper_common
from . import helper_aws_entrypoint
from .helper_parameters import *  # NOQA
from multithreader import threads


def aws_sso_update():
    """Update AWS SSO User and Group names from mappings files."""
    # Enter AWS environment.
    aws = helper_aws_entrypoint.enter_aws()
    identity_store = aws['session'].client(
        'identitystore',
        region_name=region
    )

    # Prepare items for multithreading.
    items = {
        'identity_store': identity_store,
        'identity_store_id': identity_store_id,
        'DRY_RUN': DRY_RUN
    }

    # Read User CSV file and execute updates with multithreading.
    if (
        sso_update_user_map_file
        and sso_update_old_user_heading
        and sso_update_new_user_heading
    ):
        user_data = helper_common.read_csv(sso_update_user_map_file)
        user_dicts = [
            {
                'old_user_name': row[sso_update_old_user_heading],
                'new_user_name': row[sso_update_new_user_heading]
            } for row in user_data
        ]
        user_updates = threads(
            helper_aws_sso.update_user_name,
            user_dicts,
            items,
            thread_num=THREAD_NUM
        )
        pp(user_updates)

    # Read Group CSV file and execute updates with multithreading.
    if (
        sso_update_group_map_file
        and sso_update_old_group_heading
        and sso_update_new_group_heading
    ):
        group_data = helper_common.read_csv(sso_update_group_map_file)
        group_dicts = [
            {
                'old_group_name': row[sso_update_old_group_heading],
                'new_group_name': row[sso_update_new_group_heading]
            } for row in group_data
        ]
        group_updates = threads(
            helper_aws_sso.update_group_name,
            group_dicts,
            items,
            thread_num=THREAD_NUM
        )
        pp(group_updates)


def main():
    """Execute main function."""
    aws_sso_update()


if __name__ == '__main__':
    main()



from . import helper_aws_entrypoint as helper_aws_entrypoint, helper_common as helper_common

def get_sgs(account_id: str, items: dict) -> dict: ...
def get_vpcs(account_id: str, items: dict) -> dict: ...
def main() -> None: ...



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Provide common AWS EC2 multithreaded functions."""

from . import helper_common
from . import helper_aws_entrypoint


def get_sgs(
    account_id: str,
    items: dict
) -> dict:
    """Get AWS Security Group details from an AWS account."""
    print(f'Working on {account_id}...')

    try:
        # Get auth credentials for each account.
        client = helper_aws_entrypoint.account_client_auth(
            account_id,
            items,
            'ec2'
        )

        # Get all Security Group details in the specified region.
        paginator = client.get_paginator('describe_security_groups')
        response_iterator = paginator.paginate()

        # Read through pages of Security Group details.
        sg_list = []
        for page in response_iterator:
            for sg in page['SecurityGroups']:
                sg_list.append(sg)

        # Process details.
        if sg_list == []:
            number_of_sgs = 0
            sgs = None
        else:
            number_of_sgs = len(sg_list)
            sgs = [sg['GroupId'] for sg in sg_list]

    # In case of any exceptions, return the error message but allow loop to continue.
    except Exception as e:
        number_of_sgs = 0
        sgs = [str(e)]

    return {
        'account_id': account_id,
        'number_of_sgs': number_of_sgs,
        'details': sgs
    }


def get_vpcs(
    account_id: str,
    items: dict
) -> dict:
    """Get VPC details in an AWS account."""
    print(f'Working on {account_id}...')

    try:
        # Get auth credentials for each account.
        client = helper_aws_entrypoint.account_client_auth(
            account_id,
            items,
            'ec2'
        )

        # Use a paginator to handle multiple pages of results.
        paginator = client.get_paginator('describe_vpcs')
        response_iterator = paginator.paginate()

        # Collect VPC information.
        vpcs = []
        for page in response_iterator:
            for vpc in page.get('Vpcs', []):
                vpc_tags = vpc.get('Tags', [])
                vpc_name = helper_common.get_name_from_tags(vpc_tags)
                vpc_info = {
                    'vpc_id': vpc.get('VpcId'),
                    'vpc_name': vpc_name,
                    'cidr_block': vpc.get('CidrBlock'),
                    'is_default': vpc.get('IsDefault', False)
                }
                vpcs.append(vpc_info)

    except Exception as e:
        vpcs = [
            {
                'error': str(e)
            }
        ]

    return {
        'account_id': f"'{account_id}",
        'vpcs': vpcs
    }


def main():
    """Execute main function."""
    pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Provide AWS auth and crawler options."""

from .helper_parameters import *  # NOQA
import boto3
import aws_crawler
from aws_authenticator import AWSAuthenticator as awsauth


def auth(
    auth_method: str,
    profile_name: str,
    access_key_id: str,
    secret_access_key: str,
    sso_url: str,
    sso_role_name: str,
    sso_account_id: str
):
    """Login to AWS."""
    if auth_method == 'profile':
        auth = awsauth(
            profile_name=profile_name
        )
        session = auth.profile()
    elif auth_method == 'iam':
        auth = awsauth(
            access_key_id=access_key_id,
            secret_access_key=secret_access_key
        )
        session = auth.iam()
    elif auth_method == 'sso':
        auth = awsauth(
            sso_url=sso_url,
            sso_role_name=sso_role_name,
            sso_account_id=sso_account_id
        )
        session = auth.sso()
    else:
        raise ValueError('auth_method is not defined.')
    return session


def crawler(
    session,
    accounts_list_src: str,
    accounts: str,
    ou: str,
    statuses: str
) -> list:
    """Create AWS account list."""
    if accounts_list_src == 'accounts':
        account_ids = aws_crawler.create_account_list(
            accounts
        )
    elif accounts_list_src == 'ou':
        account_ids = aws_crawler.list_ou_accounts(
            session,
            ou
        )
    elif accounts_list_src == 'statuses':
        account_ids = aws_crawler.list_accounts(
            session,
            statuses
        )
    else:
        raise ValueError('accounts_list_src is not defined.')
    return account_ids


def enter_aws() -> dict:
    """Enter AWS environment."""
    session = auth(
        auth_method=auth_method,
        profile_name=profile_name,
        access_key_id=access_key_id,
        secret_access_key=secret_access_key,
        sso_url=sso_url,
        sso_role_name=sso_role_name,
        sso_account_id=sso_account_id
    )
    account_ids = crawler(
        session,
        accounts_list_src,
        accounts,
        ou,
        statuses
    )
    return {
        'session': session,
        'account_ids': account_ids
    }


def account_client_auth(
    account_id: str,
    items: dict,
    service: str
):
    """Authenticate and return AWS client for an account."""
    credentials = aws_crawler.get_credentials(
        items['session'],
        f'arn:aws:iam::{account_id}:role/{items["assumed_role_name"]}',
        items['external_id']
    )
    client = boto3.client(
        service,
        aws_access_key_id=credentials['aws_access_key_id'],
        aws_secret_access_key=credentials['aws_secret_access_key'],
        aws_session_token=credentials['aws_session_token'],
        region_name=items['region']
    )
    return client


def main():
    """Execute main function."""
    pass


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: latin-1 -*-
"""Collect AWS Security Group details from multiple accounts."""

from . import helper_aws_ec2
from . import helper_common
from . import helper_aws_entrypoint
from .helper_parameters import *  # NOQA
from multithreader import threads


def aws_sgs_get():
    """Collect AWS Security Group details."""
    # Enter AWS environment.
    aws = helper_aws_entrypoint.enter_aws()

    # Execute task with multithreading.
    items = {
        'session': aws['session'],
        'assumed_role_name': assumed_role_name,
        'external_id': external_id,
        'region': region
    }
    results = threads(
        helper_aws_ec2.get_sgs,
        aws['account_ids'],
        items,
        thread_num=THREAD_NUM
    )

    # Convert results to CSV data and write to file.
    csv_data = helper_common.dicts_to_csv(results)
    helper_common.export_csv(csv_data, sgs_get_output_file)


def main():
    """Execute main function."""
    aws_sgs_get()


if __name__ == '__main__':
    main()



from .helper_parameters import *

def auth(auth_method: str, profile_name: str, access_key_id: str, secret_access_key: str, sso_url: str, sso_role_name: str, sso_account_id: str): ...
def crawler(session, accounts_list_src: str, accounts: str, ou: str, statuses: str) -> list: ...
def enter_aws() -> dict: ...
def account_client_auth(account_id: str, items: dict, service: str): ...
def main() -> None: ...
